@@ -25,18 +25,20 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <ctype.h>
+#include <sys/types.h>
+#include <utime.h>
+#include <time.h>
 
 #include <glib.h>
 #include <glib/gprintf.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 
-#include <dbus/dbus-glib-bindings.h>
-
 #include <libtracker-miner/tracker-miner.h>
+#include <libtracker-common/tracker-file-utils.h>
+#include <libtracker-common/tracker-date-time.h>
 
 #include "tracker-albumart.h"
-#include "tracker-dbus.h"
 #include "tracker-extract.h"
 #include "tracker-marshal.h"
 #include "tracker-albumart-generic.h"
@@ -51,15 +53,17 @@ typedef struct {
 	gchar *local_uri;
 } GetFileInfo;
 
-static void albumart_queue_cb (DBusGProxy     *proxy,
-                               DBusGProxyCall *call,
-                               gpointer        user_data);
 
 static gboolean initialized;
 static gboolean disable_requests;
 static TrackerStorage *albumart_storage;
 static GHashTable *albumart_cache;
-static DBusGProxy *albumart_proxy;
+static GDBusConnection *connection;
+
+static void
+albumart_queue_cb (GObject      *source_object,
+                   GAsyncResult *res,
+                   gpointer      user_data);
 
 static gboolean
 albumart_strip_find_next_block (const gchar    *original,
@@ -100,16 +104,16 @@ albumart_strip_find_next_block (const gchar    *original,
 static gchar *
 albumart_strip_invalid_entities (const gchar *original)
 {
-	GString         *str_no_blocks;
-	gchar          **strv;
-	gchar           *str;
-	gboolean         blocks_done = FALSE;
-	const gchar     *p;
-	const gchar     *invalid_chars = "()[]<>{}_!@#$^&*+=|\\/\"'?~";
-	const gchar     *invalid_chars_delimiter = "*";
-	const gchar     *convert_chars = "\t";
-	const gchar     *convert_chars_delimiter = " ";
-	const gunichar   blocks[5][2] = {
+	GString *str_no_blocks;
+	gchar **strv;
+	gchar *str;
+	gboolean blocks_done = FALSE;
+	const gchar *p;
+	const gchar *invalid_chars = "()[]<>{}_!@#$^&*+=|\\/\"'?~";
+	const gchar *invalid_chars_delimiter = "*";
+	const gchar *convert_chars = "\t";
+	const gchar *convert_chars_delimiter = " ";
+	const gunichar blocks[5][2] = {
 		{ '(', ')' },
 		{ '{', '}' },
 		{ '[', ']' },
@@ -177,11 +181,13 @@ albumart_strip_invalid_entities (const gchar *original)
 	str = g_strjoinv (convert_chars_delimiter, strv);
 	g_strfreev (strv);
 
-	/* Now remove double spaces */
-	strv = g_strsplit (str, "  ", -1);
-	g_free (str);
-	str = g_strjoinv (" ", strv);
-	g_strfreev (strv);
+	while (g_strrstr (str, "  ") != NULL) {
+		/* Now remove double spaces */
+		strv = g_strsplit (str, "  ", -1);
+		g_free (str);
+		str = g_strjoinv (" ", strv);
+		g_strfreev (strv);
+	}
 
 	/* Now strip leading/trailing white space */
 	g_strstrip (str);
@@ -252,6 +258,8 @@ albumart_get_path (const gchar  *artist,
 	artist_down = g_utf8_strdown (artist_stripped, -1);
 	album_down = g_utf8_strdown (album_stripped, -1);
 
+	/* g_print ("[%s] [%s]\n", artist_down, album_down); */
+
 	g_free (artist_stripped);
 	g_free (album_stripped);
 
@@ -329,6 +337,7 @@ albumart_heuristic (const gchar *artist,
 	gchar *artist_stripped = NULL;
 	gchar *album_stripped = NULL;
 	gchar *artist_strdown, *album_strdown;
+	guint i;
 
 	if (copied) {
 		*copied = FALSE;
@@ -417,81 +426,91 @@ albumart_heuristic (const gchar *artist,
 	artist_strdown = artist_stripped ? g_utf8_strdown (artist_stripped, -1) : g_strdup ("");
 	album_strdown = album_stripped ? g_utf8_strdown (album_stripped, -1) : g_strdup ("");
 
-	/* Try to find cover art in the directory */
-	for (name = g_dir_read_name (dir), count = 0, retval = FALSE;
-	     name != NULL && !retval && count < 50;
-	     name = g_dir_read_name (dir), count++) {
-		gchar *name_utf8, *name_strdown;
+	for (retval = FALSE, i = 0; i < 2 && !retval; i++) {
 
-		name_utf8 = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
+		/* We loop twice to find secondary choices, or just once as soon as a
+		 * primary choice is found. When i != 0 it means that we're in the
+		 * secondary choice's loop (current only switch is 'large' vs. 'small') */
 
-		if (!name_utf8) {
-			g_debug ("Could not convert filename '%s' to UTF-8", name);
-			continue;
-		}
+		g_dir_rewind (dir);
 
-		name_strdown = g_utf8_strdown (name_utf8, -1);
+		/* Try to find cover art in the directory */
+		for (name = g_dir_read_name (dir), count = 0, retval = FALSE;
+		     name != NULL && !retval && count < 50;
+		     name = g_dir_read_name (dir), count++) {
+			gchar *name_utf8, *name_strdown;
 
-		/* Accept cover, front, folder, AlbumArt_{GUID}_Large
-		 * reject AlbumArt_{GUID}_Small and AlbumArtSmall
-		 */
-		if ((artist_strdown && artist_strdown[0] != '\0' && strstr (name_strdown, artist_strdown)) ||
-		    (album_strdown && album_strdown[0] != '\0' && strstr (name_strdown, album_strdown)) ||
-		    (strstr (name_strdown, "cover")) ||
-		    (strstr (name_strdown, "front")) ||
-		    (strstr (name_strdown, "folder")) ||
-		    ((strstr (name_strdown, "albumart") && strstr (name_strdown, "large")))) {
-			if (g_str_has_suffix (name_strdown, "jpeg") ||
-			    g_str_has_suffix (name_strdown, "jpg")) {
-				if (!target) {
-					albumart_get_path (artist_stripped,
-					                   album_stripped,
-					                   "album",
-					                   NULL,
-					                   &target,
-					                   NULL);
-				}
+			name_utf8 = g_filename_to_utf8 (name, -1, NULL, NULL, NULL);
 
-				if (!file && target) {
-					file = g_file_new_for_path (target);
-				}
+			if (!name_utf8) {
+				g_debug ("Could not convert filename '%s' to UTF-8", name);
+				continue;
+			}
 
-				if (file) {
-					GFile *found_file;
+			name_strdown = g_utf8_strdown (name_utf8, -1);
+
+			/* Accept cover, front, folder, AlbumArt_{GUID}_Large (first choice)
+			 * second choice is AlbumArt_{GUID}_Small and AlbumArtSmall. We 
+			 * don't support just AlbumArt. (it must have a Small or Large) */
+
+			if ((artist_strdown && artist_strdown[0] != '\0' && strstr (name_strdown, artist_strdown)) ||
+			    (album_strdown && album_strdown[0] != '\0' && strstr (name_strdown, album_strdown)) ||
+			    (strstr (name_strdown, "cover")) ||
+			    (strstr (name_strdown, "front")) ||
+			    (strstr (name_strdown, "folder")) ||
+			    ((strstr (name_strdown, "albumart") && strstr (name_strdown, i == 0 ? "large" : "small")))) {
+				if (g_str_has_suffix (name_strdown, "jpeg") ||
+				    g_str_has_suffix (name_strdown, "jpg")) {
+					if (!target) {
+						albumart_get_path (artist_stripped,
+						                   album_stripped,
+						                   "album",
+						                   NULL,
+						                   &target,
+						                   NULL);
+					}
+
+					if (!file && target) {
+						file = g_file_new_for_path (target);
+					}
+
+					if (file) {
+						GFile *found_file;
+						gchar *found;
+
+						found = g_build_filename (dirname, name, NULL);
+						g_debug ("Album art (JPEG) found in same directory being used:'%s'", found);
+
+						found_file = g_file_new_for_path (found);
+						g_free (found);
+
+						retval = g_file_copy (found_file, file, 0, NULL, NULL, NULL, &error);
+						g_object_unref (found_file);
+
+						g_clear_error (&error);
+					}
+				} else if (g_str_has_suffix (name_strdown, "png")) {
 					gchar *found;
 
+					if (!target) {
+						albumart_get_path (artist_stripped,
+						                   album_stripped,
+						                   "album",
+						                   NULL,
+						                   &target,
+						                   NULL);
+					}
+
 					found = g_build_filename (dirname, name, NULL);
-					g_debug ("Album art (JPEG) found in same directory being used:'%s'", found);
-
-					found_file = g_file_new_for_path (found);
+					g_debug ("Album art (PNG) found in same directory being used:'%s'", found);
+					retval = tracker_albumart_file_to_jpeg (found, target);
 					g_free (found);
-
-					retval = g_file_copy (found_file, file, 0, NULL, NULL, NULL, &error);
-					g_object_unref (found_file);
-
-					g_clear_error (&error);
 				}
-			} else if (g_str_has_suffix (name_strdown, "png")) {
-				gchar *found;
-
-				if (!target) {
-					albumart_get_path (artist_stripped,
-					                   album_stripped,
-					                   "album",
-					                   NULL,
-					                   &target,
-					                   NULL);
-				}
-
-				found = g_build_filename (dirname, name, NULL);
-				g_debug ("Album art (PNG) found in same directory being used:'%s'", found);
-				retval = tracker_albumart_file_to_jpeg (found, target);
-				g_free (found);
 			}
-		}
 
-		g_free (name_utf8);
-		g_free (name_strdown);
+			g_free (name_utf8);
+			g_free (name_strdown);
+		}
 	}
 
 	if (count >= 50) {
@@ -549,45 +568,37 @@ albumart_request_download (TrackerStorage *storage,
                            const gchar    *local_uri,
                            const gchar    *art_path)
 {
-	GetFileInfo *info;
+	if (connection) {
+		GetFileInfo *info;
 
-	if (disable_requests) {
-		return;
-	}
-
-	info = g_slice_new (GetFileInfo);
-
-	info->storage = storage ? g_object_ref (storage) : NULL;
-
-	info->local_uri = g_strdup (local_uri);
-	info->art_path = g_strdup (art_path);
-
-	if (!albumart_proxy) {
-		GError          *error = NULL;
-		DBusGConnection *connection;
-
-		connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
-
-		if (!error) {
-			albumart_proxy = dbus_g_proxy_new_for_name (connection,
-			                                            ALBUMARTER_SERVICE,
-			                                            ALBUMARTER_PATH,
-			                                            ALBUMARTER_INTERFACE);
-		} else {
-			g_error_free (error);
+		if (disable_requests) {
+			return;
 		}
-	}
 
-	dbus_g_proxy_begin_call (albumart_proxy,
-	                         "Queue",
-	                         albumart_queue_cb,
-	                         info,
-	                         NULL,
-	                         G_TYPE_STRING, artist,
-	                         G_TYPE_STRING, album,
-	                         G_TYPE_STRING, "album",
-	                         G_TYPE_UINT, 0,
-	                         G_TYPE_INVALID);
+		info = g_slice_new (GetFileInfo);
+
+		info->storage = storage ? g_object_ref (storage) : NULL;
+
+		info->local_uri = g_strdup (local_uri);
+		info->art_path = g_strdup (art_path);
+
+		g_dbus_connection_call (connection,
+		                        ALBUMARTER_SERVICE,
+		                        ALBUMARTER_PATH,
+		                        ALBUMARTER_INTERFACE,
+		                        "Queue",
+		                        g_variant_new ("(sssu)",
+		                                       artist ? artist : "",
+		                                       album ? album : "",
+		                                       "album",
+		                                       0),
+		                        NULL,
+		                        G_DBUS_CALL_FLAGS_NONE,
+		                        -1,
+		                        NULL,
+		                        albumart_queue_cb,
+		                        info);
+	}
 }
 
 static void
@@ -661,30 +672,31 @@ albumart_copy_to_local (TrackerStorage *storage,
 }
 
 static void
-albumart_queue_cb (DBusGProxy     *proxy,
-                   DBusGProxyCall *call,
-                   gpointer        user_data)
+albumart_queue_cb (GObject      *source_object,
+                   GAsyncResult *res,
+                   gpointer      user_data)
 {
-	GError      *error = NULL;
-	guint        handle;
+	GError *error = NULL;
 	GetFileInfo *info;
+	GVariant *v;
 
 	info = user_data;
 
-	dbus_g_proxy_end_call (proxy, call, &error,
-	                       G_TYPE_UINT, &handle,
-	                       G_TYPE_INVALID);
+	v = g_dbus_connection_call_finish ((GDBusConnection *) source_object, res, &error);
 
 	if (error) {
-		if (error->code == DBUS_GERROR_SERVICE_UNKNOWN) {
+		if (error->code == G_DBUS_ERROR_SERVICE_UNKNOWN) {
 			disable_requests = TRUE;
 		} else {
 			g_warning ("%s", error->message);
 		}
-
 		g_clear_error (&error);
 	}
 
+	if (v) {
+		g_variant_unref (v);
+	}
+
 	if (info->storage && info->art_path &&
 	    g_file_test (info->art_path, G_FILE_TEST_EXISTS)) {
 
@@ -706,11 +718,12 @@ albumart_queue_cb (DBusGProxy     *proxy,
 gboolean
 tracker_albumart_init (void)
 {
-	DBusGConnection *connection;
 	GError *error = NULL;
 
 	g_return_val_if_fail (initialized == FALSE, FALSE);
 
+	tracker_albumart_plugin_init ();
+
 	albumart_storage = tracker_storage_new ();
 
 	/* Cache to know if we have already handled uris */
@@ -720,7 +733,7 @@ tracker_albumart_init (void)
 	                                        NULL);
 
 	/* Signal handler for new album art from the extractor */
-	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
+	connection = g_bus_get_sync (G_BUS_TYPE_SESSION, NULL, &error);
 
 	if (!connection) {
 		g_critical ("Could not connect to the D-Bus session bus, %s",
@@ -729,12 +742,6 @@ tracker_albumart_init (void)
 		return FALSE;
 	}
 
-	/* Get album art downloader proxy */
-	albumart_proxy = dbus_g_proxy_new_for_name (connection,
-	                                            ALBUMARTER_SERVICE,
-	                                            ALBUMARTER_PATH,
-	                                            ALBUMARTER_INTERFACE);
-
 	initialized = TRUE;
 
 	return TRUE;
@@ -745,8 +752,8 @@ tracker_albumart_shutdown (void)
 {
 	g_return_if_fail (initialized == TRUE);
 
-	if (albumart_proxy) {
-		g_object_unref (albumart_proxy);
+	if (connection) {
+		g_object_unref (connection);
 	}
 
 	if (albumart_cache) {
@@ -757,9 +764,21 @@ tracker_albumart_shutdown (void)
 		g_object_unref (albumart_storage);
 	}
 
+	tracker_albumart_plugin_shutdown ();
+
 	initialized = FALSE;
 }
 
+static void
+set_mtime (const gchar *filename, guint64 mtime)
+{
+
+	struct utimbuf buf;
+
+	buf.actime = buf.modtime = mtime;
+	utime (filename, &buf);
+}
+
 gboolean
 tracker_albumart_process (const unsigned char *buffer,
                           size_t               len,
@@ -769,9 +788,10 @@ tracker_albumart_process (const unsigned char *buffer,
                           const gchar         *filename)
 {
 	gchar *art_path;
-	gboolean processed = TRUE;
+	gboolean processed = TRUE, a_exists, created = FALSE;
 	gchar *local_uri = NULL;
 	gchar *filename_uri;
+	guint64 mtime, a_mtime = 0;
 
 	g_debug ("Processing album art, buffer is %ld bytes, artist:'%s', album:'%s', filename:'%s', mime:'%s'",
 	         (long int) len,
@@ -780,11 +800,13 @@ tracker_albumart_process (const unsigned char *buffer,
 	         filename,
 	         mime);
 
-	if (strstr (filename, "://")) {
-		filename_uri = g_strdup (filename);
-	} else {
-		filename_uri = g_filename_to_uri (filename, NULL, NULL);
-	}
+	/* TODO: We can definitely work with GFiles better here */
+
+	filename_uri = (strstr (filename, "://") ?
+	                g_strdup (filename) :
+	                g_filename_to_uri (filename, NULL, NULL));
+
+	mtime = tracker_file_get_mtime_uri (filename_uri);
 
 	albumart_get_path (artist,
 	                   album,
@@ -802,66 +824,76 @@ tracker_albumart_process (const unsigned char *buffer,
 		return FALSE;
 	}
 
-	if (!g_file_test (art_path, G_FILE_TEST_EXISTS)) {
-		/* If we have embedded album art */
-		if (buffer && len > 0) {
-			processed = albumart_set (buffer,
-			                          len,
-			                          mime,
-			                          artist,
-			                          album,
-			                          filename_uri);
-		} else {
-			/* If not, we perform a heuristic on the dir */
-			gchar *key;
-			gchar *dirname = NULL;
-			GFile *file, *dirf;
+	a_exists = g_file_test (art_path, G_FILE_TEST_EXISTS);
 
-			file = g_file_new_for_uri (filename_uri);
-			dirf = g_file_get_parent (file);
-			if (dirf) {
-				dirname = g_file_get_path (dirf);
-				g_object_unref (dirf);
-			}
-			g_object_unref (file);
+	if (a_exists) {
+		a_mtime = tracker_file_get_mtime (art_path);
+	}
 
-			key = g_strdup_printf ("%s-%s-%s",
-			                       artist ? artist : "",
-			                       album ? album : "",
-			                       dirname ? dirname : "");
+	if ((buffer && len > 0) && ((!a_exists) || (a_exists && mtime > a_mtime))) {
+		processed = albumart_set (buffer,
+		                          len,
+		                          mime,
+		                          artist,
+		                          album,
+		                          filename_uri);
+		set_mtime (art_path, mtime);
+		created = TRUE;
+	}
 
-			g_free (dirname);
+	if ((!created) && ((!a_exists) || (a_exists && mtime > a_mtime))) {
+		/* If not, we perform a heuristic on the dir */
+		gchar *key;
+		gchar *dirname = NULL;
+		GFile *file, *dirf;
 
-			if (!g_hash_table_lookup (albumart_cache, key)) {
-				if (!albumart_heuristic (artist,
-				                         album,
-				                         filename_uri,
-				                         local_uri,
-				                         NULL)) {
-					/* If the heuristic failed, we
-					 * request the download the
-					 * media-art to the media-art
-					 * downloaders
-					 */
-					albumart_request_download (albumart_storage,
-					                           artist,
-					                           album,
-					                           local_uri,
-					                           art_path);
-				}
-
-				g_hash_table_insert (albumart_cache,
-				                     key,
-				                     GINT_TO_POINTER(TRUE));
-			} else {
-				g_free (key);
-			}
+		file = g_file_new_for_uri (filename_uri);
+		dirf = g_file_get_parent (file);
+		if (dirf) {
+			dirname = g_file_get_path (dirf);
+			g_object_unref (dirf);
 		}
+		g_object_unref (file);
 
+		key = g_strdup_printf ("%s-%s-%s",
+		                       artist ? artist : "",
+		                       album ? album : "",
+		                       dirname ? dirname : "");
+
+		g_free (dirname);
+
+		if (!g_hash_table_lookup (albumart_cache, key)) {
+			if (!albumart_heuristic (artist,
+			                         album,
+			                         filename_uri,
+			                         local_uri,
+			                         NULL)) {
+				/* If the heuristic failed, we
+				 * request the download the
+				 * media-art to the media-art
+				 * downloaders
+				 */
+				albumart_request_download (albumart_storage,
+				                           artist,
+				                           album,
+				                           local_uri,
+				                           art_path);
+			}
+
+			set_mtime (art_path, mtime);
+
+			g_hash_table_insert (albumart_cache,
+			                     key,
+			                     GINT_TO_POINTER(TRUE));
+		} else {
+			g_free (key);
+		}
 	} else {
-		g_debug ("Album art already exists for uri:'%s' as '%s'",
-		         filename_uri,
-		         art_path);
+		if (!created) {
+			g_debug ("Album art already exists for uri:'%s' as '%s'",
+			         filename_uri,
+			         art_path);
+		}
 	}
 
 	if (local_uri && !g_file_test (local_uri, G_FILE_TEST_EXISTS)) {
