@@ -286,7 +286,7 @@ get_file_urns (TrackerSparqlConnection *connection,
 	                         "  %s "
 	                         "}",
 	                         tag ? "nao:hasTag ?t ; " : "",
-	                         filter);
+	                         filter ? filter : "");
 
 	cursor = tracker_sparql_connection_query (connection, query, NULL, &error);
 
@@ -522,17 +522,15 @@ print_file_report (TrackerSparqlCursor *cursor,
 	for (i = 0; uris[i]; i++) {
 		gboolean found = FALSE;
 
-		g_print ("*** checking uri:'%s' and rewinding cursor\n", uris[i]);
+		tracker_sparql_cursor_rewind (cursor);
 
 		while (tracker_sparql_cursor_next (cursor, NULL, NULL)) {
 			const gchar *str;
 
 			str = tracker_sparql_cursor_get_string (cursor, 1, NULL);
-			g_print ("***   comparing to str:'%s'\n", str);
 
 			if (g_strcmp0 (str, uris[i]) == 0) {
 				found = TRUE;
-				g_print ("***   FOUND FOUND FOUND!\n");
 				break;
 			}
 		}
@@ -540,8 +538,6 @@ print_file_report (TrackerSparqlCursor *cursor,
 		g_print ("  %s: %s\n",
 		         found ? found_msg : not_found_msg,
 		         uris[i]);
-
-		tracker_sparql_cursor_rewind (cursor);
 	}
 }
 
@@ -551,14 +547,40 @@ add_tag_for_urns (TrackerSparqlConnection *connection,
                   const gchar             *tag,
                   const gchar             *description)
 {
-	TrackerSparqlCursor *cursor;
+	TrackerSparqlCursor *cursor = NULL;
 	GError *error = NULL;
-	GStrv  uris;
+	GStrv  uris = NULL, urns_strv = NULL;
 	gchar *tag_escaped;
 	gchar *query;
 
 	tag_escaped = get_escaped_sparql_string (tag);
 
+	if (files) {
+		uris = get_uris (files);
+
+		if (!uris) {
+			return FALSE;
+		}
+
+		cursor = get_file_urns (connection, uris, NULL);
+
+		if (!cursor) {
+			g_printerr ("Files do not exist or aren't indexed\n");
+			g_strfreev (uris);
+			return FALSE;
+		}
+
+		urns_strv = result_to_strv (cursor, 0);
+
+		if (!urns_strv || g_strv_length (urns_strv) < 1) {
+			g_printerr ("Files do not exist or aren't indexed\n");
+			g_object_unref (cursor);
+			g_strfreev (uris);
+
+			return FALSE;
+		}
+	}
+
 	if (description) {
 		gchar *description_escaped;
 
@@ -604,8 +626,15 @@ add_tag_for_urns (TrackerSparqlConnection *connection,
 		g_printerr ("%s, %s\n",
 		            _("Could not add tag"),
 		            error->message);
+
+		if (cursor) {
+			g_object_unref (cursor);
+		}
+
 		g_error_free (error);
 		g_free (tag_escaped);
+		g_strfreev (urns_strv);
+		g_strfreev (uris);
 
 		return FALSE;
 	}
@@ -613,24 +642,12 @@ add_tag_for_urns (TrackerSparqlConnection *connection,
 	g_print ("%s\n",
 	         _("Tag was added successfully"));
 
-	uris = get_uris (files);
-
-	if (!uris) {
-		/* No URIs to tag */
-		g_free (tag_escaped);
-		return TRUE;
-	}
-
-	cursor = get_file_urns (connection, uris, NULL);
-
 	/* First we check if the tag is already set and only add if it
 	 * is, then we add the urns specified to the new tag.
 	 */
-	if (cursor) {
-		GStrv urns_strv;
+	if (urns_strv) {
 		gchar *filter;
 
-		urns_strv = result_to_strv (cursor, 0);
 		filter = get_filter_string (urns_strv, TRUE, NULL);
 
 		/* Add tag to specific urns */
@@ -643,7 +660,7 @@ add_tag_for_urns (TrackerSparqlConnection *connection,
 		                         "  %s "
 		                         "}",
 		                         tag_escaped,
-		                         filter);
+		                         filter ? filter : "");
 
 		tracker_sparql_connection_update (connection, query, 0, NULL, &error);
 		g_strfreev (urns_strv);
@@ -654,22 +671,25 @@ add_tag_for_urns (TrackerSparqlConnection *connection,
 			g_printerr ("%s, %s\n",
 			            _("Could not add tag to files"),
 			            error->message);
+			g_object_unref (cursor);
 			g_error_free (error);
 			g_free (tag_escaped);
-			g_object_unref (cursor);
+			g_strfreev (uris);
 
 			return FALSE;
 		}
 
 		print_file_report (cursor, uris, _("Tagged"),
 		                   _("Not tagged, file is not indexed"));
-
-		g_object_unref (cursor);
 	}
 
 	g_strfreev (uris);
 	g_free (tag_escaped);
 
+	if (cursor) {
+		g_object_unref (cursor);
+	}
+
 	return TRUE;
 }
 
@@ -758,7 +778,7 @@ remove_tag_for_urns (TrackerSparqlConnection *connection,
 		                         "  ?urn nao:hasTag ?t . "
 		                         "  %s "
 		                         "}",
-		                         filter);
+		                         filter ? filter : "");
 		g_free (filter);
 
 		g_object_unref (tag_cursor);
@@ -917,7 +937,7 @@ main (int argc, char **argv)
 		g_thread_init (NULL);
 	}
 
-	connection = tracker_sparql_connection_get (&error);
+	connection = tracker_sparql_connection_get (NULL, &error);
 
 	if (!connection) {
 		g_printerr ("%s: %s\n",
